#!/usr/bin/env ruby
#
# Copyright 2008 Matthew Lee Hinman
#
# A POC fuzzy stream matcher, does absolutely no ordering.
# 
# Usage: ./fuzzymatch.rb <pcapfile>

require './pcapparser.rb'
require './streambuilder.rb'

## The variation number to use
v = 5

p = PcapParser.new(File.new(ARGV[0]))

pcount = 0

## Our array of streams
$slist = []

## Fuzzy match, based on sequence number distance
def fuzzymatch(pkt, v)
  sn = pkt.seq_num.to_i
  an = pkt.ack_num.to_i
  $slist.each { |str|
    str.sa_list.each { |num|
      ## If the sequence number is close enough
      if (sn.to_i - num.to_i).abs <= v
        ## Add the packet to the stream
        puts "seq num: #{sn} close enough to #{num} to add."
        str.add_packet(pkt)
        return
      ## Maybe the ack number is close enough?
      elsif ((an.to_i - num.to_i).abs <= v) && (an != 0)
        puts "ack num: #{an} close enough to #{num} to add."
        str.add_packet(pkt)
        return
      end
    }
  }
  puts "No stream found for packet, starting a new one..."
  s = Stream.new(pkt.ip_src,pkt.ip_dst)
  s.add_packet(pkt)
  $slist.push(s)
end

p.each { |pkt|
  pcount = pcount + 1
  next unless pkt.tcp?

  print "[#{pcount}]\t[#{pkt.readable_flags}] "
  print [pkt.ip_src].pack("N").unpack("C4").join(".")
  print " -> "
  print [pkt.ip_dst].pack("N").unpack("C4").join(".")
  print "\tseq=#{pkt.seq_num} ack=#{pkt.ack_num} len=#{pkt.length}"
  print "\n"

  fuzzymatch(pkt, v)

}

puts "Ended up with #{$slist.length} stream(s)."
i = 0
$slist.each { |s|
  i = i + 1
  puts "Stream #{i} contains #{s.pkt_num} packet(s)"
}
